import { TextChannel } from 'eris';
import { v4 as genUUID } from 'uuid';
import { RichEmbed, Route, Server } from '../../../class';

export default class Root extends Route {
  constructor(server: Server) {
    super(server);
    this.conf = {
      path: '/',
    };
  }

  public bind() {
    this.router.post('/executive-orders', async (req, res) => {
      if (!req.body.pin) {
        return res.status(401).json({
          code: this.constants.codes.UNAUTHORIZED,
          message: this.constants.messages.UNAUTHORIZED,
        });
      }

      const director = await this.server.client.db.Score.findOne({ pin: req.body.pin });
      const staffGuild = this.server.client.guilds.get('446067825673633794') || await this.server.client.getRESTGuild('446067825673633794');

      if (!director || !staffGuild.members.get(director.userID)?.roles?.includes('662163685439045632')) {
        return res.status(401).json({
          code: this.constants.codes.UNAUTHORIZED,
          message: this.constants.messages.UNAUTHORIZED,
        });
      }

      if (!req.body.subject) {
        return res.status(400).json({
          code: this.constants.codes.CLIENT_ERROR,
          message: this.constants.messages.CLIENT_ERROR,
        });
      }

      if (!req.body.body) {
        return res.status(400).json({
          code: this.constants.codes.CLIENT_ERROR,
          message: this.constants.messages.CLIENT_ERROR,
        });
      }

      const executiveOrder = await this.server.client.db.ExecutiveOrder.create({
        issuedBy: director.userID,
        subject: req.body.subject,
        body: req.body.body,
        at: new Date(),
        oID: genUUID(),
      });

      const staffDiscord = this.server.client.users.get(director.userID) || await this.server.client.getRESTUser(director.userID);
      const staffInformation = await this.server.client.db.Staff.findOne({ userID: director.userID });

      const embed = new RichEmbed();
      embed.setTitle('Executive Order');
      embed.setAuthor(`${staffDiscord.username}#${staffDiscord.discriminator}, ${staffInformation.pn.join(', ')}`, staffDiscord.avatarURL);
      embed.setColor('#dd3acd');
      embed.addField('Subject', req.body.subject);
      embed.addField('Body', req.body.body);
      embed.addField('ID', executiveOrder.oID);
      embed.setTimestamp(new Date());

      const channel = <TextChannel>this.server.client.getChannel('807444198969835550');
      await channel.createMessage({ embed });

      return res.status(200).json({
        code: this.constants.codes.SUCCESS,
        message: `Created new Executive Order with ID ${executiveOrder.oID} by ${staffDiscord.username}#${staffDiscord.discriminator}, ${staffInformation.pn.join(', ')}.`,
      });
    });

    this.router.post('/motions', async (req, res) => {
      if (!req.body.pin) {
        return res.status(401).json({
          code: this.constants.codes.UNAUTHORIZED,
          message: this.constants.messages.UNAUTHORIZED,
        });
      }

      const director = await this.server.client.db.Score.findOne({ pin: req.body.pin });
      const staffGuild = this.server.client.guilds.get('446067825673633794') || await this.server.client.getRESTGuild('446067825673633794');

      if (!director || !staffGuild.members.get(director.userID)?.roles?.includes('662163685439045632')) {
        return res.status(401).json({
          code: this.constants.codes.UNAUTHORIZED,
          message: this.constants.messages.UNAUTHORIZED,
        });
      }

      if (!req.body.subject) {
        return res.status(400).json({
          code: this.constants.codes.CLIENT_ERROR,
          message: this.constants.messages.CLIENT_ERROR,
        });
      }

      if (!req.body.body) {
        return res.status(400).json({
          code: this.constants.codes.CLIENT_ERROR,
          message: this.constants.messages.CLIENT_ERROR,
        });
      }

      const motionID = genUUID();

      const staffDiscord = this.server.client.users.get(director.userID) || await this.server.client.getRESTUser(director.userID);
      const staffInformation = await this.server.client.db.Staff.findOne({ userID: director.userID });

      const embed = new RichEmbed();
      embed.setTitle('Motion');
      embed.setAuthor(`${staffDiscord.username}#${staffDiscord.discriminator}, ${staffInformation.pn.join(', ')}`, staffDiscord.avatarURL);
      embed.setColor('#66e1ff');
      embed.addField('Subject', req.body.subject);
      embed.addField('Body', req.body.body);
      embed.addField('ID', motionID);
      embed.setTimestamp(new Date());

      const channel = <TextChannel>this.server.client.getChannel('807444198969835550');
      const motionMessage = await channel.createMessage({ embed });
      await motionMessage.addReaction(this.server.client.util.emojis.SUCCESS);
      await motionMessage.addReaction(this.server.client.util.emojis.ERROR);

      const motion = await this.server.client.db.Motion.create({
        issuedBy: director.userID,
        subject: req.body.subject,
        body: req.body.body,
        at: new Date(),
        oID: motionID,
        motionMessage: motionMessage.id,
        processed: false,
      });

      return res.status(200).json({
        code: this.constants.codes.SUCCESS,
        message: `Created new Motion with ID ${motion.oID} by ${staffDiscord.username}#${staffDiscord.discriminator}, ${staffInformation.pn.join(', ')}.`,
      });
    });
  }
}
